package pl.tph.javatacka.game;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.EventQueue;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.event.KeyEvent;
import java.awt.font.FontRenderContext;
import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;

import java.lang.reflect.InvocationTargetException;
import java.util.Arrays;

import pl.tph.javatacka.graphics.ScreenManager;
import pl.tph.javatacka.input.VirtualKey;
import pl.tph.javatacka.input.InputManager;
import pl.tph.javatacka.game.sprites.Curve;
import pl.tph.javatacka.game.sprites.Score;

/**
 * Klasa <code>GameCore</code> jest gwn klas gry <em>Javatacka</em>
 * spajajc wszystkie jej elementy. To w jej metodach znajduj si procedury
 * potrzebne do inicjacji gry, gwna ptla gry oraz procedury koczce gr.
 * <br><br>
 * Ta klasa jest z pewnoci najbardziej skomplikowana pord wszystkich klas
 * nalecych do pakietw <em>pl.tph.javatacka.*</em>. Mona by jej zarzuci, e
 * cz metod mogaby zosta przeniesiona do innych klas, gdy uprocioby to
 * <code>GameCore'a</code>. Jednake odbyoby si to kosztem czasu przekazywania
 * referencji do wielu obiektw, co mogoby si bardzo niekorzystnie odbi na
 * wydajnoci gry. Z tego te powodu zosta zastosowany taki, a nie inny ukad
 * metod klasy <code>GameCore</code>.
 *
 * @author Tomasz Pieczerak (tph)
 *
 */
public class GameCore {

	/**
	 * Metoda statyczna <code>main()</code>, od ktrej swoje dziaanie rozpoczyna
	 * gwny wtek gry.
	 */
	public static void main ( String[] args ) {
		GameCore game = new GameCore();
		game.run();
	}

	/**
	 * Ta metoda uruchamia gr uprzednio inicjujc wszelkie potrzebne elementy.
	 */
	private void run ( ) {
		initGUI();

		while ( true ) {
			runMenu();
			
			try {
				initGame();
				gameLoop();
			} finally {
				screen.restoreScreen();
			}
		}
	}

	/**
	 * Inicjuje wszelkie elementy skadajce si na <em>Graficzny Interfejs
	 * Uytkownika</em>.
	 */
	private void initGUI ( ) {
		screen = new ScreenManager();
		
		try {
			EventQueue.invokeAndWait(new Runnable()
				{
					@Override
					public void run ( ) { menu = new GameMenu(); }
				});
		} catch (InterruptedException e1) { }
		  catch (InvocationTargetException e1) { }
	}

	/**
	 * Wywietla okno menu gry i czeka, a gracz ustawi opcje gry i klikn
	 * przycisk <i>Start!</i> (aby rozpocz gr) lub <i>Wyjcie</i>, aby j
	 * zakoczy.
	 */
	private void runMenu ( ) {
		menu.setDone(false);
		menu.setVisible(true);
		
		while ( !(menu.isDone()) ) {
			try {
				Thread.sleep(100);
			} catch (InterruptedException e) { }
		}
		menu.setVisible(false);
	}

	/**
	 * Inicjuje wszystkie elementy potrzebne do waciwej czci rozgrywki.
	 */
	private void initGame ( ) {
		screen.setFullScreen("Javatacka");
		screen.getFullScreenWindow().requestFocus();

		gameField = new BufferedImage(screen.getWidth()-100, screen.getHeight(),
									  BufferedImage.TYPE_BYTE_GRAY);
		
		input = new InputManager(screen.getFullScreenWindow());
		quit = new VirtualKey(VirtualKey.INITIAL_PRESS_ONLY);
		input.mapToKey(quit, KeyEvent.VK_ESCAPE);
		pause = new VirtualKey(VirtualKey.INITIAL_PRESS_ONLY);
		input.mapToKey(pause, KeyEvent.VK_SPACE);

		loadPreferences(menu.getGamePreferences());
	}

	/**
	 * Na podstawie otrzymanego obiektu <code>GamePreferences</code> ustawia
	 * odpowiednie opcje w obiekcie <code>GameCore</code>.
	 * @param pref obiekt zawierajcy preferencje gry
	 */
	private void loadPreferences ( GamePreferences pref ) {
		players = new Player[pref.getPlayerAmount()];
		maxScore = pref.getMaxScore();

		// warto tego sleep'a zostaa wyznaczona empirycznie, jednak oglnie
		// ma by tak, e im wiksza prdko gry tym krtsze sleep'y
		sleep = (int) Math.round(1f/Math.pow(pref.getSpeed(),2) * 13500f);
		if ( sleep < 5 ) sleep = 5;
		else if ( sleep > 30 ) sleep = 25;

		for( int i = 0; i < pref.getPlayerAmount(); ++i) {
			players[i] = pref.getPlayer(i, input);
			players[i].getCurve().setVelocity(0.005f * pref.getSpeed());
		}
	}

	/**
	 * Metoda zawierajca gwn ptl waciwej gry.
	 */
	private void gameLoop ( ) {
		long timeFps = 0;
		int frameCounter = 0;

		long elapsedTime = 0;
		long waitTime = 0;
		long currTime = System.currentTimeMillis();
		
		gameState = GAME_NEW_ROUND;

		for ( Player player : players ) {
			player.getScore().setXY(screen.getWidth()+100, 100);
			player.getScore().setGotoPositionX(screen.getWidth()-95);
		}

		while ( true ) { // gwna ptla gry

			elapsedTime = System.currentTimeMillis() - currTime;
			currTime += elapsedTime;

			if ( timeFps < 1000 ) {
				++frameCounter;
				timeFps += elapsedTime;
			}
			else {
				fps = frameCounter;
				timeFps = 0;
				frameCounter = 0;
			}

			if ( gameState != GAME_OVER && quit.isPressed() )
				endGame();

			// GAME_NEW_ROUND - rozpocznij now rund
			if ( gameState == GAME_NEW_ROUND ) {
				newRound();
				gameState = GAME_READY;
			}
			// GAME_READY - czekaj na rozpoczcie rozgrywki
			else if ( gameState == GAME_READY ) {
				if( pause.isPressed() )
					gameState = GAME_RUNNING;

				gameWait();
				update(elapsedTime);
				draw();
			}
			// GAME_RUNNING - w trakcie rozgrywki
			else if ( gameState == GAME_RUNNING ) {
				update(elapsedTime);
				checkCollisions();
				draw();

				if( pause.isPressed() )
					gameState = GAME_PAUSE;

				if ( alivePlayers < 2 ) {
					gameState = GAME_END_ROUND;
					waitTime = 0;
				}
			}
			// GAME_PAUSE - pauza w rozgrywce
			else if ( gameState == GAME_PAUSE ) {
				update(elapsedTime);
				draw();

				if( pause.isPressed() )
					gameState = GAME_RUNNING;

			}
			// GAME_END_ROUND - zakocz rund
			else if ( gameState == GAME_END_ROUND ) {
				waitTime += elapsedTime;

				update(elapsedTime);
				draw();

				if ( waitTime > 2000 || pause.isPressed() ) {
					if ( players[0].getScoreValue() >= maxScore )
						endGame();
					else
						gameState = GAME_NEW_ROUND;
				}
			}
			// GAME_OVER - zakocz gr i poka wyniki graczy
			else if ( gameState == GAME_OVER ) {
				if( pause.isPressed() || quit.isPressed() )
					break;

				for ( Player player : players )
					player.getScore().update(elapsedTime);

				draw();
			}

			// odpocznij chwil, eby nie obcia za bardzo procesora
			try {
				Thread.sleep(sleep);
			} catch ( InterruptedException ex ) {	}
		}
	}

	/**
	 * Metoda inicjujca now rund.
	 */
	private void newRound ( ) {
		Graphics2D g2 = gameField.createGraphics();

		g2.setColor(Color.BLACK);
		g2.fill(new Rectangle(screen.getWidth()-100, screen.getHeight()));
		g2.dispose();

		Curve curve;
		for ( Player player : players ) {
			player.setAlive(true);
			player.resetKeys();
			
			curve = player.getCurve();
			curve.setDirection((double)Math.random() * 360);
			curve.setXY(Math.random() * (screen.getWidth() - 400)  + 150,
						Math.random() * (screen.getHeight() - 300) + 150);
			curve.setOldXY(curve.getX(), curve.getY());
		}
		alivePlayers = players.length;
	}

	/**
	 * Metoda wywoywana podczas oczekiwania na rozpoczcie rundy.
	 */
	private void gameWait ( ) {
		Graphics2D g2 = screen.getGraphics();

		g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
							RenderingHints.VALUE_ANTIALIAS_ON);
		g2.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,
							RenderingHints.VALUE_TEXT_ANTIALIAS_ON);

		g2.setColor(Color.BLACK);
		g2.fill(new Rectangle(screen.getWidth(), screen.getHeight()));

		if ( gameState == GAME_READY ) {
			Curve curve;
			for ( Player player : players ) {
				curve = player.getCurve();

				g2.setColor(curve.getColor());
				g2.fillOval((int)Math.round(curve.getX())-2,
							(int)Math.round(curve.getY())-2, 5, 5);
			}

			Font font = new Font("Monospaced", Font.BOLD, 20);
			FontRenderContext context = g2.getFontRenderContext();
			Rectangle2D bounds = font.getStringBounds("Press SPACE to start!",
													  context);
			g2.setColor(Color.WHITE);
			g2.setFont(font);
			g2.drawString("Press SPACE to start!",
					(int)((screen.getWidth() - bounds.getWidth()) / 2), 50);
		}
		
		g2.dispose();
	}

	/**
	 * Gwna funkcja aktualizujca wszystkie obiekty gry. Cz z nich
	 * (te poruszajce si po ekranie) jest aktualizowana na podstawie podanego
	 * czasu.
	 * @param elapsedTime czas, ktry min od ostatniej aktualizacji
	 */
	private void update ( long elapsedTime ) {
		Arrays.sort(players);

		for ( int i = 0; i < players.length ; ++i ) {
			players[i].getScore().setGotoPositionY(Score.playerPosition[i]+20);
			players[i].getScore().update(elapsedTime);

			if ( gameState == GAME_RUNNING && players[i].isAlive() )
				players[i].update(elapsedTime);
		}
	}

	/**
	 * Metoda realizujca detekcj kolizji midzy <em>krzywymi</em> graczy.
	 */
	private void checkCollisions ( ) {
		double checkX;
		double checkY;
		Curve curve;
		Color color;
		Graphics2D g2;

		for ( Player player : players ) {
			if ( player.isAlive() ) {
				g2 = screen.getGraphics();
				curve = player.getCurve();

				checkX = curve.getX();
				checkY = curve.getY();

				// przy wolnych prdkociach musimy uwaa, eby nie sprawdzi
				// kolizji w miejscu, gdzie znajduje si "gowa" naszej krzywej
				while (	Math.abs(checkX - curve.getOldX()) < 3.0 &&
						Math.abs(checkY - curve.getOldY()) < 3.0 )
				{
					checkX += curve.getVelocityX() * 2;
					checkY += curve.getVelocityY() * 2;
				}

				// sprawdzamy w ptli kolory wszystkich pikseli midzy starym
				// i nowym pooeniem "gowy" krzywej
				do
				{
					try {
						color = new Color(
									gameField.getRGB((int)Math.round(checkX),
													 (int)Math.round(checkY)));

						// jeli pobrany kolor jest kolorem ta, to oznacza, to
						// e wszystko jest ok, wic przechodzimy do kolejnego
						// piksela skaczc na koniec ptli
						if ( color.getRed()   == 0 &&
							 color.getGreen() == 0 &&
							 color.getBlue()  == 0)
						{	
							checkX -= curve.getVelocityX() * 2;
							checkY -= curve.getVelocityY() * 2;
							continue; // wszystko ok, wic pomijamy procedur
									  // zabijania gracza skaczc na koniec
						}
					} catch ( ArrayIndexOutOfBoundsException e ) {
						// jeli podane wsprzdne wychodziy po za pole gry,
						// to oznacza, i gracz wyjecha po za nie, czyli
						// zderzy si ze cian
					}

					// procedura zabijania gracza, ktry uderzy w przeszkod
					player.setAlive(false);
					--alivePlayers;

					// rysowanie ostatniego odcinka krzywej
					curve.setXY(checkX, checkY);
					player.getCurve().draw(g2);
					g2.dispose();

					// zwiksz liczb punktw yjcym graczom
					for ( Player p : players ) {
						if ( p.isAlive() )
							p.incrementScoreValue();
					}

					// gracz ju nie gra, wic moemy zakoczy sprawdzanie
					break;
				}
				while ( Math.abs(checkX - curve.getOldX()) > 3.0 ||
						Math.abs(checkY - curve.getOldY()) > 3.0 );

				// aktualizacja zapisu planszy w buforze do sprawdzania kolizji
				g2 = (Graphics2D) gameField.getGraphics();
				player.getCurve().rawDraw(g2);
		        g2.dispose();
			}
		}
	}

	/**
	 * Gwna procedura rysujca wszystkie elementy na ekranie.
	 */
	private void draw ( ) {
		Graphics2D g2 = screen.getGraphics();
		Graphics2D field = (Graphics2D) gameField.getGraphics();
		
		g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
							RenderingHints.VALUE_ANTIALIAS_ON);
		g2.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,
							RenderingHints.VALUE_TEXT_ANTIALIAS_ON);

		if ( gameState == GAME_OVER ) {
			g2.setColor(Color.BLACK);
			g2.fill(new Rectangle(screen.getWidth(), screen.getHeight()));
			
			Font font = new Font("Monospaced", Font.BOLD, 30);
			FontRenderContext context = g2.getFontRenderContext();
			Rectangle2D bounds = font.getStringBounds("Game Over", context);

			g2.setColor(Color.WHITE);
			g2.setFont(font);
			g2.drawString("Game Over",
					(int)((screen.getWidth() - bounds.getWidth()) / 2),
					Math.round(screen.getHeight()/10));
			
			font = new Font("Monospaced", Font.PLAIN, 11);
			context = g2.getFontRenderContext();
			bounds = font.getStringBounds("Press ESC or SPACE to exit",
										  context);

			g2.setFont(font);
			g2.drawString("Press ESC or SPACE to exit",
					(int)((screen.getWidth() - bounds.getWidth()) / 2),
					Math.round(screen.getHeight()-20));
		}
		else {
			g2.setStroke(new BasicStroke(1));
			g2.setColor(Color.BLACK);
			g2.fillRect(screen.getWidth()-100, 1, 100, screen.getHeight()-1);
			g2.setColor(Color.WHITE);
			g2.drawRect(screen.getWidth()-100, 1, 99, screen.getHeight()-2);

			g2.setFont(new Font("Monospaced", Font.PLAIN, 11));
			g2.drawString("FPS:"+fps, screen.getWidth()-80, screen.getHeight()-20);
		}

		if ( gameState != GAME_OVER && gameState != GAME_READY) {
			AffineTransform at = new AffineTransform();
			at.rotate(-Math.PI/2);
			Font font = new Font("Monospaced", Font.PLAIN, 11);

			g2.setFont(font.deriveFont(at));
			g2.drawString("Press SPACE to toggle pause", screen.getWidth()-10,
														 screen.getHeight()-10);
		}

		for ( Player player : players ) {
			player.getScore().draw(g2);
			
			if ( gameState == GAME_RUNNING && player.isAlive() ) {
				player.getCurve().draw(g2);
				player.getCurve().rawDraw(g2);
			}
		}

		//g2.drawImage(gameField, 0, 0, screen.getFullScreenWindow());
		g2.dispose();
        field.dispose();
		screen.update();
	}

	/**
	 * Metoda suca do koczenia rozgrywki.
	 */
	private void endGame ( ) {
		Arrays.sort(players);

		for ( int i = 0; i < players.length ; ++i ) {
			players[i].setAlive(true);
			players[i].getScore().setGotoPositionX((screen.getWidth()/2)-45);
			players[i].getScore().setGotoPositionY(
				Score.playerPosition[i]+Math.round(screen.getHeight()*0.16f));
		}
		gameState = GAME_OVER;
	}


	/**
	 * Menader okna penoekranowego
	 * @see pl.tph.javatacka.graphics.ScreenManager
	 */
	private ScreenManager screen;
	/**
	 * Menader wejcia z klawiatury i myszy.
	 * @see pl.tph.javatacka.input.InputManager
	 */
	private InputManager input;
	/**
	 * Menu gry.
	 * @see pl.tph.javatacka.game.GameMenu
	 */
	private GameMenu menu;
	
	/**
	 * Aktualny stan gry. Jest to nie jako stan gry jako automatu skoczonego.
	 */
	private int gameState;
	private static final int GAME_NEW_ROUND	= 0;
	private static final int GAME_READY		= 1;
	private static final int GAME_RUNNING	= 2;
	private static final int GAME_PAUSE		= 3;
	private static final int GAME_END_ROUND	= 4;
	private static final int GAME_OVER		= 5;

	/**
	 * Uproszczony czarno-biay zapis pola gry. Uywany do detekcji kolizji.
	 */
	private BufferedImage gameField;
	/**
	 * Tabela zawierajca obiekty graczy.
	 * @see pl.tph.javatacka.game.Player
	 */
	private Player players[];
	/**
	 * Liczba <i>"ywych"</i> graczy biorcy jeszcze udzia w danej rundzie.
	 */
	private int alivePlayers;
	/**
	 * Limit punktw potrzebny do zwycistwa.
	 */
	private int maxScore;

	/**
	 * Akcja gry - wyjcie z gry.
	 * @see pl.tph.javatacka.input.VirtualKey
	 */
	private VirtualKey quit;
	/**
	 * Akcja gry - pauza.
	 * @see pl.tph.javatacka.input.VirtualKey
	 */
	private VirtualKey pause;

	/**
	 * Czas na jaki wtek zasypia po kadym obiegu gwnej ptli gry.
	 */
	private int sleep;
	private int fps;
}
