package pl.tph.javatacka.game;

import java.awt.Color;
import java.awt.event.KeyEvent;

import pl.tph.javatacka.game.sprites.Curve;
import pl.tph.javatacka.game.sprites.Score;
import pl.tph.javatacka.input.InputManager;
import pl.tph.javatacka.input.VirtualKey;

/**
 * Obiekty klasy <code>Player</code> przechowuj w sobie wszelkie potrzebne
 * informacje o graczu. Po za takimi rzeczami jak nazwa graczy czy aktualna
 * liczba punktw znajduj si tu take rne obiekty bezporednio biorce
 * udzia w rozgrywce lub wpywajce na ni.
 * <br><br>
 * Dodatkowo klasa <code>Player</code> implementuje interfejs
 * <code>Comparable</code> dziki obiekty tej klasy mog by atwo sortowane
 * i porwnywane.
 * 
 * @author Tomasz Pieczerak (tph)
 *
 */
public class Player implements Comparable<Object> {

	/**
	 * Konstruuje nowy obiekt klasy <code>Player</code> na podstawie podanych
	 * argumentw. Jednoczenie zostan utworzone obiekty <code>Curve</code> i
	 * <code>Score</code> zwizane z tym graczem.
	 * @param name nazwa gracza
	 * @param color kolor <em>krzywej</em> gracza
	 * @param leftKeyCode kod lewego klawisza sterowania
	 * @param rightKeyCode kod prawego klawisza sterowania
	 * @param input obiekt <code>InputManager</code> obsugujcy zdarzenia
	 * wcinicia klawiszy sterujcych przez gracza
	 * @see java.awt.Color
	 * @see java.awt.event.KeyEvent
	 * @see pl.tph.javatacka.game.sprites.Score
	 * @see pl.tph.javatacka.game.sprites.Curve
	 */
	public Player ( String name, Color color,
					int leftKeyCode, int rightKeyCode, InputManager input)
	{
		this.name = name;
		this.scoreValue = 0;
		this.alive = true;
		this.curve = new Curve(color);
		this.score = new Score(this);

		this.leftKey = new VirtualKey();
		this.rightKey = new VirtualKey();

		if ( leftKeyCode == 1 && rightKeyCode == 3 ) {
			input.mapToMouse(leftKey, InputManager.MOUSE_BUTTON_1);
			input.mapToMouse(rightKey, InputManager.MOUSE_BUTTON_3);
			keyString = "Mouse";
		}
		else {
			input.mapToKey(leftKey, leftKeyCode);
			input.mapToKey(rightKey, rightKeyCode);
			
			// w zaoeniu klawisze s wybierana z ograniczonego zbioru par,
			// zatem na podstawie lewego klawisza mona te ustali prawy
			switch ( leftKeyCode ) {
			case KeyEvent.VK_1:
				keyString =  "1 Q";
				break;
			case KeyEvent.VK_CONTROL:
				keyString =  "Ctrl Win";
				break;
			case KeyEvent.VK_C:
				keyString =  "C V";
				break;
			case KeyEvent.VK_M:
				keyString =  "M ,";
				break;
			case KeyEvent.VK_LEFT:
				keyString =  "Left Down";
				break;
			default:
				keyString = "Unknown";
			}
		}
	}

	/**
	 * Uaktualnia obiekt gracza, a dokadniej obiekt <em>krzywej</em> na
	 * podstawie minionego czasu. Modyfikowany jest kierunek poruszania, ktry
	 * mg ulec zmianie w skutek wcinicia klawiszy. Nastpnie uaktualniany
	 * jest sam obiekt <code>Curve</code>.
	 * @param elapsedTime miniony czas
	 * @see pl.tph.javatacka.game.sprites.Curve
	 */
	public void update ( long elapsedTime ) {
		if ( leftKey.isPressed() )
			curve.setDirection(curve.getDirection()+(((double)elapsedTime)/4f)*
					(curve.getVelocity()/0.15f));
		if ( rightKey.isPressed() )
			curve.setDirection(curve.getDirection()-(((double)elapsedTime)/4f)*
					(curve.getVelocity()/0.15f));

		curve.update(elapsedTime); 
	}

	/**
	 * Metoda implementujca odziedziczon metod z interfejsu
	 * <code>Comparable</code>. Gracze s porwnywani wedug liczby zdobytych
	 * punktw (im wicej punktw tym <i>wikszy</i>) obiekt. Jeli dwaj gracze
	 * maj po tyle samo punktw, to wikszy bdzie ten, ktry jest aktualnie
	 * <i>"ywy"</i>. Jeli obaj yj, to s sobie rwni w sensie tego porzdku. 
	 */
	@Override
	public int compareTo ( Object o ) {
		if ( !(o instanceof Player) )
			throw new ClassCastException("A Player object expected.");

		int score = ((Player)o).getScoreValue();
		boolean alive = ((Player)o).isAlive();

		if ( score > this.scoreValue )
			return 1;
		else if ( score < this.scoreValue )
			return -1;
		else if ( alive != this.alive ) {
			if ( alive == true )
				return 1;
			else
				return -1;
		}
		else
			return 0;
	}

	/**
	 * Wymusza zwolnienie obu <em>klawiszy</em> sterowania gracza.
	 * @see pl.tph.javatacka.input.VirtualKey
	 */
	public void resetKeys ( ) {
		leftKey.release();
		rightKey.release();
	}

	/**
	 * Zwraca nazw gracza.
	 */
	public String getName ( ) { return name; }
	/**
	 * Zwraca referencj na <em>krzyw</em> gracza.
	 */
	public Curve getCurve ( ) {	return curve; }
	/**
	 * Zwraca referencj na obiekt tabelki z wynikami gracza.
	 */
	public Score getScore ( ) { return score; }
	/**
	 * Zwraca mnemoniczny odpowiednik klawiszy sterowania gracza.
	 */
	public String getKeyString ( ) { return keyString; }

	/**
	 * Ustawia warto flagi mwicej o tym, czy gracz <i>yje</i> czy jest
	 * <i>martwy</i>.
	 * @param alive nowa warto flagi
	 */
	public void setAlive ( boolean alive ) { this.alive = alive; }
	/**
	 * Zwraca flaga mwica o tym czy gracz jest aktualnie <i>"ywy"</i>
	 * czy nie.
	 */
	public boolean isAlive ( ) { return alive; }

	/**
	 * Zwraca aktualn liczb punktw zdobytych przez gracza. 
	 */
	public int getScoreValue ( ) { return scoreValue; }
	/**
	 * Inkrementuje o jedne liczb punktw zdobytych przez gracza.
	 */
	public void incrementScoreValue ( ) { ++scoreValue;	}

	/**
	 * Nazwa gracza.
	 */
	private String name;
	/**
	 * Obiekt <em>krzywej</em> gracza.
	 * @see pl.tph.javatacka.game.sprites.Curve
	 */
	private Curve curve;
	/**
	 * Obiekt tabelki z wynikami gracza.
	 * @see pl.tph.javatacka.game.sprites.Score
	 */
	private Score score;
	/**
	 * Aktualna liczba punktw zdobytych przez gracza w danej rozgrywce.
	 */
	private int scoreValue;
	/**
	 * Flaga mwica o tym czy gracz jest aktualnie <i>"ywy"</i>, czyli czy w
	 * danej rundzie ju przegra czy nie.
	 */
	private boolean alive;

	/**
	 * Obiekt akcji gry zwizanej z wciniciem przez gracza lewego klawisza
	 * sterowania. Jego wcinicie powoduje zmian kierunku ruchu
	 * <em>krzywej</em> przeciwnie do ruchu wskazwek zegara.
	 */
	private VirtualKey leftKey;
	/**
	 * Obiekt akcji gry zwizanej z wciniciem przez gracza prawego klawisza
	 * sterowania. Jego wcinicie powoduje zmian kierunku ruchu
	 * <em>krzywej</em> zgodnie z ruchem wskazwek zegara.
	 */
	private VirtualKey rightKey;
	/**
	 * Napis reprezentujcy mnemoniczny odpowiednik klawiszy sterowania gracza.
	 * Jest on wywietlany na ekranie w obiekcie <code>Score</code> w celach
	 * informacyjnych - eby gracz nie zapomnia, ktrymi klawiszami steruje.
	 * @see pl.tph.javatacka.game.sprites.Score
	 */
	private String keyString;
}
