package pl.tph.javatacka.game.sprites;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;

import pl.tph.javatacka.graphics.Sprite;

/**
 * Obiekty Klasy <code>Curve</code> reprezentuj <em>krzywe</em> kontrolowane
 * przez graczy. S one najistotniejszym elementem dynamicznym w grze.
 * <br><br>
 * Klasa <code>Curve</code> rozszerza klas abstrakcyjn <code>Sprite</code>
 * dodajc do niej nowe pola i metody potrzebne do reprezentacji
 * <em>krzywych</em>. W tych obiektach przechowywane s informacje o aktualnym
 * i ostatnim pooeniu <i>"gowy"</i> <em>krzywej</em>, wartoci jej wektora
 * prdkoci, kierunku poruszania oraz jej kolorze.<br>
 * Co moe dziwi w obiekcie nie s przechowywane adne informacje o samej
 * krzywej (w sensie geometrycznym).
 * <br><br>
 * Krzywe rysowane na ekranie nie s cige. Co jaki czas mog si w nich
 * losowo pojawia przerwy o rwnie losowej dugoci.
 *   
 * @author Tomasz Pieczerak (tph)
 * @see pl.tph.javatacka.graphics.Sprite
 */
public class Curve extends Sprite {

	/**
	 * Konstruktor klasy <code>Curve</code> tworzy now krzyw o podanym
	 * kolorze. Wszystkie pozostae pola s inicjowane wartociami zerowymi.
	 * @param color
	 */
	public Curve ( Color color ) {
		super(0, 0, 0, 0, 4, 4);
		this.oldX = 0;
		this.oldY = 0;
		this.velocity = 0;
		this.direction = 0;
		this.hole = false;
		this.lastHoleDistance = 0f;
		this.color = color;
	}

	/**
	 * Aktualizuje pozycj <i>"gowy"</i> <em>krzywej</em> w oparciu o warto
	 * wektora prdkoci oraz kierunek poruszania.
	 */
	@Override
	public void update ( long elapsedTime ) {
		// wylicz skadow pionow i poziom wektora prdkoci
		dx = (double) velocity * Math.sin(Math.toRadians(direction));
		dy = (double) velocity * Math.cos(Math.toRadians(direction));
		// zapamitaj ostanie pooenie
		oldX = x;
		oldY = y;
		// ustaw nowe pooenie w oparciu o miniony czas
		x += dx * elapsedTime;
		y += dy * elapsedTime;

		// ustal aktualnie jest dziura czy nie
		if ( hole ) {
			if ( (Math.random() + velocity - 0.15f) * lastHoleDistance > 5 ) {
				hole = false;
				lastHoleDistance = 0f;
			}
			else {
				lastHoleDistance +=
					Math.sqrt( Math.pow((x-oldX), 2) + Math.pow((y-oldY), 2) );
			}
		}
		else {
			if ( Math.random() * lastHoleDistance > 300 ) {
				hole = true;
				lastHoleDistance = 0f;
			}
			else {
				lastHoleDistance +=
					Math.sqrt( Math.pow((x-oldX), 2) + Math.pow((y-oldY), 2) );
			}
		}
	}

	/**
	 * Rysuje krzyw w odpowiednim kolorze w podanym kontekcie graficznym.
	 * @param g2 kontekst graficzny
	 */
	public void draw ( Graphics2D g2 ) {
		g2.setColor(color);
		rawDraw(g2);
	}

	/**
	 * Rysuje krzyw w podanym kontekcie graficznym bez ustawiania koloru.
	 * @param g2 kontekst graficzny
	 */
	public void rawDraw ( Graphics2D g2 ) {
		if ( !hole ) {
			g2.setStroke(new BasicStroke(width));
			g2.drawLine((int) Math.round(x), (int) Math.round(y),
						(int) Math.round(oldX), (int) Math.round(oldY));
		}
	}

	/**
	 * Ustawia aktualn pozycj <i>"gowy"</i> <em>krzywej</em>.
	 * @param x pionowa skadowa pooenia
	 * @param y pozioma skadowa pooenia
	 */
	public void setXY ( double x, double y ) {
		this.x = x;
		this.y = y;
	}
	/**
	 * Ustawia poprzedni pozycj <i>"gowy"</i> <em>krzywej</em>.
	 * @param oldX pionowa skadowa pooenia
	 * @param oldY pozioma skadowa pooenia
	 */
	public void setOldXY ( double oldX, double oldY ) {
		this.oldX = oldX;
		this.oldY = oldY;
	}

	/**
	 * Zwraca aktualn warto wektora prdkoci. Prdko jest podana w
	 * pikselach na milisekund.
	 */
	public double getVelocity ( ) { return velocity; }
	/**
	 * Ustawia warto wektora prdkoci na podan
	 * @param velocity szybko w pikselach na milisekund
	 */
	public void setVelocity ( double velocity ) { this.velocity = velocity; }

	/**
	 * Zwraca aktualny kierunek poruszania si <em>krzywej</em> w stopniach.
	 */
	public double getDirection ( ) { return direction; }
	/**
	 * Ustawia kierunek poruszania si <em>krzywej</em> na podany
	 * @param direction kierunek w stopniach (moe by ponad 360)
	 */
	public void setDirection ( double direction ) {
		this.direction = direction%360;
	}

	/**
	 *  Zwraca kolor krzywej.
	 */
	public Color getColor ( ) { return color; }
	/**
	 * Zwraca pionow skadow poprzedniego pooenia.
	 */
	public double getOldX ( ) { return oldX; }
	/**
	 * Zwraca poziom skadow poprzedniego pooenia.
	 */
	public double getOldY ( ) { return oldY; }
	/**
	 * Zwraca aktualn warto pionowej skadowej wektora prdkoci.
	 */
	public double getVelocityX ( ) { return dx; }
	/**
	 * Zwraca aktualn warto poziomej skadowej wektora prdkoci.
	 */
	public double getVelocityY ( ) { return dy; }
	/**
	 * Zwraca informacj czy aktualnie w krzywej jest dziura.
	 */
	public boolean isHole ( ) { return hole; }

	/**
	 * Pionow skadowa poprzedniego pooenia.
	 */
	private double oldX;
	/**
	 * Pozioma skadowa poprzedniego pooenia.
	 */
	private double oldY;

	/**
	 * Aktualna warto wektora prdkoci
	 */
	private double velocity;
	/**
	 * Aktualny kierunek poruszania si wyraony w stopniach.
	 */
	private double direction;

	/**
	 * Flaga informujca czy aktualnie w krzywej jest dziura.
	 */
	private boolean hole;
	/**
	 * Dystans od ostatniej zmiany z dziura na nie-dziura lub na odwrt.
	 */
	private double lastHoleDistance;

	/**
	 * Kolor krzywej.
	 */
	private Color color;
}
