package pl.tph.javatacka.game.sprites;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;

import pl.tph.javatacka.graphics.Sprite;
import pl.tph.javatacka.game.Player;

/**
 * Obiekty klasy <code>Score</code> odzwierciedlaj tabelki graczy pokazujce
 * aktualn liczb punktw zdobytych przez danego gracza w biecej rozgrywce.
 * <br><br>
 * Klasa <code>Score</code> jest podklas abstrakcyjnej klasy
 * <code>Sprite</code>.
 * <br><br>
 * Najwaniejsz dodan funkcjonalnoci s dwie metody:
 * <code>setGotoPositionX(int)</code> oraz <code>setGotoPositionY(int)</code>,
 * ktre uatwiaj sterowanie poruszaniem si tych obiektw po ekranie.
 * Wywoujc je z odpowiednimi parametrami ustawiamy pozycj - kolejno jej
 * skadow poziom lub pionow - do ktrej obiekt <code>Score</code> bdzie
 * dy w swoim ruchu na ekranie. Jest to zrealizowane przez automatyczn
 * modyfikuj wektor prdkoci tak, aby po pewnym czasie obiekt zatrzyma si na
 * owej ustalonej pozycji. Zosta tu zastosowany prosty <em>regulator typu P
 * </em> o duym wzmocnieniu, co dziki dyskretnoci pooenia na ekranie daje
 * szybkie i pynne osignicie zadanej pozycji przy zerowym
 * <em>przeregulowaniu</em> oraz zerowym <em>uchybie statycznym</em>.
 *
 * @author Tomasz Pieczerak (tph)
 * @see pl.tph.javatacka.graphics.Sprite
 * @see <a href="http://pl.wikipedia.org/wiki/Regulator_P">regulator typu P</a>
 */
public class Score extends Sprite {
	/**
	 * Tabela <code>playerPosition[]</code> zawiera pionowe skadowe pooenia
	 * dla szeciu pozycji graczy, ktre zapewniaj dobry rozkad tabelek
	 * <code>Score</code> na ekranie. Naley jest stosowa jako wartoci bazowe,
	 * do ktrym mona doda wasne przesunicie, aby ustawi zestawienie
	 * tabelek z wynikami w wybranym miejscu na ekranie. 
	 */
	public static final int[] playerPosition = {0, 70, 140, 210, 280, 350};

	/**
	 * Tworzy nowy obiekt <code>Score</code> inicjujc jego skadowe pooenia,
	 * wektora prdkoci i pooenia, do ktrego on dy wartociami zerowymi.
	 * Utworzony obiekt jest wizany z podanym obiektem <code>Player</code>, z
	 * ktrego bdzie pobiera nazw, liczb punktw oraz kolor krzywej gracza 
	 * (obiekt klasy <code>Curve</code>).
	 * @param player gracz, z ktrym ma by zwizany obiekt <code>Score</code>
	 * @see pl.tph.javatacka.game.Player
	 * @see pl.tph.javatacka.game.sprites.Curve
	 */
	public Score ( Player player ) {
		super(0, 0, 0, 0, 90, 60);
		this.goToPositionX = 0;
		this.goToPositionY = 0;
		this.player = player;
	}

	/**
	 * Aktualizuje pozycj obiektu <code>Score</code> na ekranie w oparciu po
	 * podany jako argument miniony czas. Wektor prdkoci jest ustalany tak,
	 * aby obiekt porusza si na ekranie w kierunku zadanej pozycji, a do
	 * zatrzymania na niej.
	 */
	@Override
	public void update ( long elapsedTime ) {
		if ( Math.abs(goToPositionY - getY()) < 1 )
			setVelocityY(0);
		else
			setVelocityY(0.01f * (goToPositionY - getY()) );
	
		if ( Math.abs(goToPositionX - getX()) < 1 )
			setVelocityX(0);
		else
			setVelocityX(0.01f * (goToPositionX - getX()) );

		x += dx * elapsedTime;
		y += dy * elapsedTime;
	}

	/**
	 * Rysuje obiekt <code>Score</code> na komponencie zwizanym z podanym
	 * obiektem <code>Graphics2D</code>.
	 * @param g2 kontekst graficzny zwizany z komponentem, na ktrym
	 * <code>Score</code> ma zosta narysowany.
	 * @see java.awt.Graphics2D
	 */
	public void draw ( Graphics2D g2 ) {
		g2.setStroke(new BasicStroke(2));

		if ( player.isAlive() )
			g2.setColor(Color.WHITE);
		else
			g2.setColor(Color.GRAY);

		g2.drawRoundRect((int)getX(),(int)getY(),getWidth(),getHeight(),20,20);

		g2.setFont(new Font("Monospaced", Font.PLAIN, 11));
		g2.drawString(player.getName(), (int)getX()+10, (int)getY() + 15);

		g2.setFont(new Font("Monospaced", Font.BOLD, 13));
		g2.drawString(Integer.toString(player.getScoreValue()),
					  (int)getX() + 50, (int)getY() + 51);

		g2.setColor(player.getCurve().getColor());
		g2.fillRoundRect((int)getX()+10,(int)getY()+20, 70, 15, 6, 6);

		g2.setColor(Color.BLACK);
		g2.setFont(new Font("Monospaced", Font.BOLD, 10));
		g2.drawString(player.getKeyString(),(int)getX()+15, (int)getY()+31);
	}

	/**
	 * Przenosi obiekt <code>Score</code> w podane miejsce na ekranie. Pozycja,
	 * do ktrej on dy rwnie zostaje ustawiona na ten punkt.
	 * @param x pozioma wsprzdna miejsca na ekranie
	 * @param y pionowa wsprzdna miejsca na ekranie
	 */
	public void setXY ( double x, double y ) {
		this.x = x;
		this.y = y;
		this.goToPositionX = (int)Math.round(x);
		this.goToPositionY = (int)Math.round(y);
	}

	/**
	 * Ustawia pozioma skadow pooenia, do ktrego w swoim ruchu obiekt
	 * <code>Score</code> bdzie dy.
	 * @param goToPositionX pozioma skadowa pooenia
	 */
	public void setGotoPositionX ( int goToPositionX ) {
		this.goToPositionX = goToPositionX;
	}

	/**
	 * Ustawia pionowa skadow pooenia, do ktrego w swoim ruchu obiekt
	 * <code>Score</code> bdzie dy.
	 * @param goToPositionY pionowa skadowa pooenia
	 */
	public void setGotoPositionY ( int goToPositionY ) {
		this.goToPositionY = goToPositionY;
	}

	/**
	 * Referencja na obiekt gracza, z ktrym zwizany jest <code>Score</code>.
	 * @see pl.tph.javatacka.game.Player
	 */
	private Player player;
	
	/**
	 * Pozioma skadowa pooenia na ekranie, do ktrego dy obiekt
	 * <code>Score</code> (w pikselach).
	 */
	private int goToPositionX;
	/**
	 * Pionowa skadowa pooenia na ekranie, do ktrego dy obiekt
	 * <code>Score</code> (w pikselach).
	 */
	private int goToPositionY;
}
