package pl.tph.javatacka.graphics;

import java.awt.Graphics2D;
import java.awt.GraphicsDevice;
import java.awt.GraphicsEnvironment;
import java.awt.Toolkit;
import java.awt.Window;
import java.awt.image.BufferedImage;

import javax.swing.JFrame;

/**
 * Klasa <code>ScreenManager</code> suy do inicjalizacji i zarzdzania oknem
 * pracujcym w trybie penoekranowym. Wywietlanie obrazu w oknie implementuje
 * mechanizm podwjnego buforowania, ktry zapewnia stabilne wywietlanie
 * wszelkiej grafiki.
 *
 * @author Tomasz Pieczerak (tph)
 *
 */
public class ScreenManager {

	/**
	 * Tworzy nowy obiekt zarzdcy penoekranowego okna.
	 */
	public ScreenManager ( ) {
		GraphicsEnvironment env = 
				GraphicsEnvironment.getLocalGraphicsEnvironment();
		this.device = env.getDefaultScreenDevice();
	}

	/**
	 * Przechodzi w tryb penoekranowy. Ta metoda tworzy nowe okno o podanym
	 * tytule i ustawia je jako penoekranowe. Nie jest natomiast zmieniany
	 * aktualny tryb graficzny (rozdzielczo, gbia kolorw oraz
	 * czstotliwo odwieania ekranu). Podwjne buforowanie zrealizowane jest
	 * przez proste i przenonie przeczanie stron (<i>page-flipping</i>).
	 * @param title tytu penoekranowego okna
	 */
	public void setFullScreen ( String title ) {
		final JFrame frame = new JFrame();

		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setUndecorated(true);
		frame.setIgnoreRepaint(true);
		frame.setResizable(false);
		frame.setTitle(title);

		device.setFullScreenWindow(frame);

		buffer = new BufferedImage(getWidth(), getHeight(),
								   BufferedImage.TYPE_INT_RGB);
	}

	/**
	 * Zwraca obiekt <code>Graphics2D</code> dla uruchomionego
	 * okna penoekranowego. 
	 */
	public Graphics2D getGraphics ( ) {
		return (Graphics2D) buffer.getGraphics();
	}

	/**
	 * Po zakoczeniu rysowania w oknie za pomoc zwizanego z nim obiektu
	 * <code>Graphics2D</code> naley uaktualni wywietlany obraz. To wanie
	 * robi ta metoda.
	 */
	public void update ( ) {
		Window win = device.getFullScreenWindow();

		Graphics2D g2 = (Graphics2D) win.getGraphics();
		g2.drawImage(buffer, 0, 0, win);
		g2.dispose();

		Toolkit.getDefaultToolkit().sync();
	}

	/**
	 * Zwraca referencj na aktualnie dziaajce okno penoekranowe. Jeli w
	 * danym momencie okno nie dziaa w trybie penoekranowym zostanie zwrcony
	 * <code>null</code>.
	 */
	public JFrame getFullScreenWindow ( ) {
		return (JFrame)device.getFullScreenWindow();
	}

	/**
	 * Ta metoda zwraca szeroko okna dziaajcego obecnie w trybie
	 * penoekranowym albo warto 0, gdy takie okno nie istnieje.
	 */
	public int getWidth ( ) {
		Window win = device.getFullScreenWindow();

		if ( win != null )
			return win.getWidth();
		else
			return 0;
	}

	/**
	 * Ta metoda zwraca wysoko okna dziaajcego obecnie w trybie
	 * penoekranowym albo warto 0, gdy takie okno nie istnieje.
	 */
	public int getHeight ( ) {
		Window win = device.getFullScreenWindow();

		if ( win != null )
			return win.getHeight();
		else
			return 0;
	}

	/**
	 * Usuwa obecnie dziaajce okno i opuszcza tryb penoekranowy.
	 */
	public void restoreScreen ( ) {
		Window win = device.getFullScreenWindow();

		if ( win != null )
			win.dispose();

		device.setFullScreenWindow(null);
	}

	/**
	 * Obiekt reprezentujcy urzdzenie graficznie zwizane z lokalnym
	 * rodowiskiem graficznym.
	 */
	private GraphicsDevice device;

	private BufferedImage buffer;
}
