package pl.tph.javatacka.input;

import java.awt.Component;
import java.awt.Cursor;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

/**
 * Klasa <code>InputManager</code> suy do zarzdzania zdarzeniami zwizanymi
 * z wejciem klawiatury i myszy (wciskanie i zwalnianie klawiszy/przyciskw).
 * Wykorzystuje ona obiekty klasy <code>VirtualKey</code> do informowania
 * innych obiektw o zaistniaych zdarzeniach.
 * 
 * @author Tomasz Pieczerak (tph)
 * @see pl.tph.javatacka.input.VirtualKey
 */
public class InputManager implements KeyListener, MouseListener {

	/**
	 * Kod lewego klawisza myszy.
	 */
	public static final int MOUSE_BUTTON_1 = 0;
	/**
	 * Kod rodkowego klawisza myszy
	 */
	public static final int MOUSE_BUTTON_2 = 1;
	/**
	 * Kod prawego klawisza myszy.
	 */
	public static final int MOUSE_BUTTON_3 = 2;

	/**
	 * Tworzy nowy obiekt klasy <code>InputManager</code>, ktry bdzie
	 * nasuchiwa zdarze przekazywanych pod podanego komponentu.
	 * @param comp komponent, z ktrego zdarzenia bd obsugiwane
	 */
	public InputManager ( Component comp ) {
		this.keyActions = Collections.synchronizedMap(
											new HashMap<Integer, VirtualKey>());
		this.mouseActions = new VirtualKey[3];

		comp.addKeyListener(this);
		comp.addMouseListener(this);
		comp.setFocusTraversalKeysEnabled(false);
		comp.setCursor(INVISIBLE_CURSOR);
	}

	/**
	 * Mapuje wskazany obiekt <code>VirtualKey</code> na wybrany klawisz (z
	 * klawiatury).
	 * @param virtualKey obiekt reprezentujcy wybran akcj w grze
	 * @param keyCode kody klawisza
	 * @see pl.tph.javatacka.input.VirtualKey
	 */
	public void mapToKey ( VirtualKey virtualKey, int keyCode ) {
		keyActions.put(keyCode, virtualKey);
	}

	/**
	 * Mapuje wskazany obiekt <code>VirtualKey</code> na wybrany przycisk myszy.
	 * @param virtualKey obiekt reprezentujcy wybran akcj w grze
	 * @param mouseCode kod klawisza myszy
	 * @see pl.tph.javatacka.input.InputManager#MOUSE_BUTTON_1
	 * @see pl.tph.javatacka.input.InputManager#MOUSE_BUTTON_2
	 * @see pl.tph.javatacka.input.InputManager#MOUSE_BUTTON_3
	 */
	public void mapToMouse ( VirtualKey virtualKey, int mouseCode ) {
		mouseActions[mouseCode] = virtualKey;
	}

	@Override
	public void keyPressed ( KeyEvent e ) {
		VirtualKey virtualKey = getKeyAction(e);

		if ( virtualKey != null )
			virtualKey.press();

		e.consume();
	}

	@Override
	public void keyReleased ( KeyEvent e ) {
		VirtualKey virtualKey = getKeyAction(e);

		if ( virtualKey != null )
			virtualKey.release();

		e.consume();
	}

	@Override
	public void keyTyped ( KeyEvent e ) { e.consume(); }

	@Override
	public void mousePressed ( MouseEvent e ) {
		VirtualKey virtualKey = getMouseButtonAction(e);

		if ( virtualKey != null )
			virtualKey.press();
	}

	@Override
	public void mouseReleased ( MouseEvent e ) {
		VirtualKey virtualKey = getMouseButtonAction(e);
		if ( virtualKey != null )
			virtualKey.release();
	}

	// ignorowane zdarzenia myszy
	@Override
	public void mouseClicked ( MouseEvent e ) { }
	@Override
	public void mouseEntered ( MouseEvent e ) { }
	@Override
	public void mouseExited  ( MouseEvent e ) { }

	/**
	 * Zwraca obiekt akcji gry <code>VirtualKey</code> zwizany z klawiszem, od
	 * ktrego przyszo zdarzenie.
	 * @param e zdarzenie wcinicia/zwolnienia klawisza z klawiatury
	 */
	private VirtualKey getKeyAction ( KeyEvent e ) {
		int keyCode = e.getKeyCode();

		if ( keyActions.containsKey (keyCode) )
			return keyActions.get(keyCode);
		else
			return null;
	}

	/**
	 * Zwraca obiekt akcji gry <code>VirtualKey</code> zwizany z przyciskiem
	 * myszy, od ktrego przyszo zdarzenie.
	 * @param e zdarzenie wcinicia/zwolnienia przycisku myszy
	 */
	private VirtualKey getMouseButtonAction ( MouseEvent e ) {
		switch ( e.getButton() ) {
		case MouseEvent.BUTTON1:
			return mouseActions[MOUSE_BUTTON_1];
		case MouseEvent.BUTTON2:
			return mouseActions[MOUSE_BUTTON_2];
		case MouseEvent.BUTTON3:
			return mouseActions[MOUSE_BUTTON_3];
		default:
			return null;
		}
	}

	/**
	 * Niewidzialny kursor myszy. Suy do ukrywania kursora na ekranie.
	 */
	private static final Cursor INVISIBLE_CURSOR =
		Toolkit.getDefaultToolkit().createCustomCursor(
				Toolkit.getDefaultToolkit().getImage(""),
				new Point(0,0), "invisible");

	/**
	 * Mapa zawierajca pary powizanych ze sob kodw klawiszy klawiatury i
	 * obiektw akcji gry <code>VirtualKey</code>.
	 */
	private Map<Integer, VirtualKey> keyActions;
	/**
	 * Tabela zawierajca obiekty akcji gry <code>VirtualKey</code> zwizane z
	 * przyciskami myszy.
	 * @see pl.tph.javatacka.input.InputManager#MOUSE_BUTTON_1
	 * @see pl.tph.javatacka.input.InputManager#MOUSE_BUTTON_2
	 * @see pl.tph.javatacka.input.InputManager#MOUSE_BUTTON_3
	 */
	private VirtualKey[] mouseActions;
}
