package pl.tph.javatacka.input;

/**
 * Klasa <code>GameAction</code> jest klas wykorzystywan przez obiekty typu
 * <code>InputManager</code> do wizania akcji inicjowanych przez uytkownika
 * z konkretnymi akcjami wejcia (np. nacinicie klawisza, ale nie koniecznie).
 * <br><br>
 * W dalszej czci dokumentacji przez <em>klawisz</em> naley rozumie pewien
 * dwustanowy obiekt, ktry moe by wcinity lub wycinity.
 * <br><br>
 * Obiekty tej klasy s synchronizowane, wic mona ich bezpiecznie uywa w
 * programach wielowtkowych.
 *
 * @author Tomasz Pieczerak (tph)
 * @see pl.tph.javatacka.input.InputManager
 */
public class VirtualKey {
	/**
	 * Normalne zachowanie klawisza. To znaczy metoda <code>isPressed()</code>
	 * zwraca <code>true</code> tak dugo jak dugo klawisz jest wcinity. 
	 */
	public static final boolean NORMAL = false;
	
	/**
	 * Z t opcj metoda <code>isPressed()</code> zwrci <code>true</code> tylko
	 * przy pierwszym wywoaniu po tym jak klawisz zosta wcinity.
	 */
	public static final boolean INITIAL_PRESS_ONLY = true;
	
	/**
	 * Tworzy nowy obiekt klasy <code>VirtualKey</code> z domylnie ustawion
	 * opcj <code>NORMAL</code>.
	 * @see pl.tph.javatacka.input.VirtualKey#INITIAL_PRESS_ONLY
	 */
    public VirtualKey ( ) { this(NORMAL); }

    /**
     * Tworzy nowy obiekt klasy <code>VirtualKey</code> z wybranym trybem
     * zachowania klawiszy.
     * @param behavior wybrany tryb zachowania klawiszy
     * @see pl.tph.javatacka.input.VirtualKey#NORMAL
     * @see pl.tph.javatacka.input.VirtualKey#INITIAL_PRESS_ONLY
     */
	public VirtualKey ( boolean behavior ) { this.behavior = behavior; }
	
	/**
	 * Wciska klawisz zwizany z tym obiektem <code>VirtualKey</code>.
	 */
	public synchronized void press ( ) {
		if ( state != STATE_WAITING_FOR_RELEASE )
			state = STATE_PRESSED;
	}
	
	/**
	 * Zwalnia klawisz zwizany z tym obiektem <code>VirtualKey</code>.
	 */
	public synchronized void release ( ) {
		state = STATE_RELEASED;
	}
	
	/**
	 * Zwraca warto logiczn mwic o tym czy klawisz zwizany z tym obiektem
	 * <code>VirtualKey</code> jest aktualnie wcinity czy nie. Zachowanie tej
	 * metody jest zgodne z wybranym trybem.
     * @see pl.tph.javatacka.input.VirtualKey#NORMAL
     * @see pl.tph.javatacka.input.VirtualKey#INITIAL_PRESS_ONLY
	 */
	public synchronized boolean isPressed ( ) {
		if ( state == STATE_PRESSED ) {
			if ( behavior == INITIAL_PRESS_ONLY )
				state = STATE_WAITING_FOR_RELEASE;
			return true;
		}
		else
			return false;
	}

	/**
	 * Klawisz zosta zwolniony.
	 */
	private static final int STATE_RELEASED = 0;
	/**
	 * Klawisz zosta wcinity.
	 */
	private static final int STATE_PRESSED = 1;
	/**
	 * Klawisz zosta wcinity, a nastpnie jego stan zosta raz sprawdzony
	 * przez metod <code>isPressed()</code>. Do momentu zwolnienia klawisza, a
	 * nastpnie jego ponownego wcinicia wywoanie tej metody zwrci
	 * <code>false</code>.  
	 */
	private static final int STATE_WAITING_FOR_RELEASE = 2;

	/**
	 * Sposb zachowania si klawisza w odpowiedzi na wywoanie metody
	 * <code>isPressed()</code>.
	 * @see pl.tph.javatacka.input.VirtualKey#isPressed
	 * @see pl.tph.javatacka.input.VirtualKey#NORMAL
     * @see pl.tph.javatacka.input.VirtualKey#INITIAL_PRESS_ONLY 
	 */
    private boolean behavior;
    /**
     * Aktualny stan klawisza.
     * @see pl.tph.javatacka.input.VirtualKey#STATE_RELEASED
     * @see pl.tph.javatacka.input.VirtualKey#STATE_PRESSED
     * @see pl.tph.javatacka.input.VirtualKey#STATE_WAITING_FOR_RELEASE
     */
    private int state;
}
